@@ -205,22 +205,27 @@ _format_value_name_with_prefix(gchar *buf, gsize buf_len,
                                JournalReaderOptions *options,
                                const gchar *key, gssize key_len)
 {
-  gsize cont = 0;
+  /* NOTE: key is usually not NUL terminated (except if we are mapping it
+   * from the journald native key to syslog-ng field name) */
 
+  gsize cont = 0;
   if (key_len < 0)
     key_len = strlen(key);
 
   if (options->prefix)
     cont = g_strlcpy(buf, options->prefix, buf_len);
-  gsize left = buf_len - cont;
+  gssize left = buf_len - cont;
   if (left >= key_len + 1)
     {
-      strncpy(buf + cont, key, key_len);
+      /* we have more than enough space in buf */
+      memcpy(buf + cont, key, key_len);
       buf[cont + key_len] = 0;
     }
-  else
+  else if (left > 0)
     {
-      g_strlcpy(buf + cont, key, buf_len - cont);
+      /* truncate key */
+      memcpy(buf + cont, key, left - 1);
+      buf[cont + left - 1] = 0;
     }
 }
 
@@ -776,7 +781,7 @@ _init(LogPipe *s)
 {
   JournalReader *self = (JournalReader *)s;
 
-#ifndef SYSLOG_NG_HAVE_JOURNAL_NAMESPACES
+#if ! SYSLOG_NG_HAVE_JOURNAL_NAMESPACES
   if (journal_reader_initialized)
     {
       msg_error("The configuration must not contain more than one systemd-journal() source");
@@ -818,7 +823,7 @@ _init(LogPipe *s)
     }
 
   self->immediate_check = TRUE;
-#ifndef SYSLOG_NG_HAVE_JOURNAL_NAMESPACES
+#if ! SYSLOG_NG_HAVE_JOURNAL_NAMESPACES
   journal_reader_initialized = TRUE;
 #endif
   _update_watches(self);
